@@ -22,10 +22,8 @@
 #define HIGHWAY_HWY_CONTRIB_SORT_TRAITS128_TOGGLE
 #endif
 
-#include <string>
-
+#include "hwy/contrib/sort/order.h"  // SortDescending
 #include "hwy/contrib/sort/shared-inl.h"
-#include "hwy/contrib/sort/vqsort.h"  // SortDescending
 #include "hwy/highway.h"
 
 HWY_BEFORE_NAMESPACE();
@@ -124,19 +122,49 @@ struct KeyAny128 {
 
 // Base class shared between OrderAscending128, OrderDescending128.
 struct Key128 : public KeyAny128 {
-  // What type to pass to Sorter::operator().
+  // False indicates the entire key should be compared. KV means key-value.
+  static constexpr bool IsKV() { return false; }
+
+  // What type to pass to VQSort.
   using KeyType = hwy::uint128_t;
 
-  std::string KeyString() const { return "U128"; }
+  const char* KeyString() const { return "U128"; }
 
   template <class D>
   HWY_INLINE Mask<D> EqualKeys(D d, Vec<D> a, Vec<D> b) const {
     return Eq128(d, a, b);
   }
 
-  HWY_INLINE bool Equal1(const LaneType* a, const LaneType* b) {
+  template <class D>
+  HWY_INLINE Mask<D> NotEqualKeys(D d, Vec<D> a, Vec<D> b) const {
+    return Ne128(d, a, b);
+  }
+
+  // For keys=entire 128 bits, any difference counts.
+  template <class D>
+  HWY_INLINE bool NoKeyDifference(D /*tag*/, Vec<D> diff) const {
+    // Must avoid floating-point comparisons (for -0)
+    const RebindToUnsigned<D> du;
+    return AllTrue(du, Eq(BitCast(du, diff), Zero(du)));
+  }
+
+  HWY_INLINE bool Equal1(const LaneType* a, const LaneType* b) const {
     return a[0] == b[0] && a[1] == b[1];
   }
+
+  // Returns vector with only the top half of each block valid. This allows
+  // fusing the "replicate upper to lower half" step with a subsequent permute.
+  template <class Order, class D>
+  HWY_INLINE HWY_MAYBE_UNUSED Vec<D> CompareTop(D d, Vec<D> a, Vec<D> b) const {
+    const Mask<D> eqHL = Eq(a, b);
+    const Vec<D> ltHL = VecFromMask(d, Order().CompareLanes(a, b));
+#if HWY_TARGET <= HWY_AVX2  // slightly faster
+    const Vec<D> ltLX = ShiftLeftLanes<1>(ltHL);
+    return OrAnd(ltHL, VecFromMask(d, eqHL), ltLX);
+#else
+    return IfThenElse(eqHL, DupEven(ltHL), ltHL);
+#endif
+  }
 };
 
 // Anything order-related depends on the key traits *and* the order (see
@@ -148,6 +176,7 @@ struct Key128 : public KeyAny128 {
 // we are anyway going to specialize at a higher level.
 struct OrderAscending128 : public Key128 {
   using Order = SortAscending;
+  using OrderForSortingNetwork = OrderAscending128;
 
   HWY_INLINE bool Compare1(const LaneType* a, const LaneType* b) {
     return (a[1] == b[1]) ? a[0] < b[0] : a[1] < b[1];
@@ -174,7 +203,7 @@ struct OrderAscending128 : public Key128 {
     return Max128(d, a, b);
   }
 
-  // Same as for regular lanes because 128-bit lanes are u64.
+  // Same as for regular lanes because 128-bit keys are u64.
   template <class D>
   HWY_INLINE Vec<D> FirstValue(D d) const {
     return Set(d, hwy::LowestValue<TFromD<D> >());
@@ -187,13 +216,18 @@ struct OrderAscending128 : public Key128 {
 
   template <class D>
   HWY_INLINE Vec<D> PrevValue(D d, Vec<D> v) const {
-    const Vec<D> k1 = OddEven(Zero(d), Set(d, 1));
-    return Sub(v, k1);
+    const Vec<D> k0 = Zero(d);
+    const Vec<D> k1 = OddEven(k0, Set(d, uint64_t{1}));
+    const Mask<D> borrow = Eq(v, k0);  // don't-care, lo == 0
+    // lo == 0? 1 : 0, 0
+    const Vec<D> adjust = ShiftLeftLanes<1>(IfThenElseZero(borrow, k1));
+    return Sub(Sub(v, k1), adjust);
   }
 };
 
 struct OrderDescending128 : public Key128 {
   using Order = SortDescending;
+  using OrderForSortingNetwork = OrderDescending128;
 
   HWY_INLINE bool Compare1(const LaneType* a, const LaneType* b) {
     return (a[1] == b[1]) ? b[0] < a[0] : b[1] < a[1];
@@ -220,7 +254,7 @@ struct OrderDescending128 : public Key128 {
     return Min128(d, a, b);
   }
 
-  // Same as for regular lanes because 128-bit lanes are u64.
+  // Same as for regular lanes because 128-bit keys are u64.
   template <class D>
   HWY_INLINE Vec<D> FirstValue(D d) const {
     return Set(d, hwy::HighestValue<TFromD<D> >());
@@ -233,30 +267,63 @@ struct OrderDescending128 : public Key128 {
 
   template <class D>
   HWY_INLINE Vec<D> PrevValue(D d, Vec<D> v) const {
-    const Vec<D> k1 = OddEven(Zero(d), Set(d, 1));
-    return Add(v, k1);
+    const Vec<D> k1 = OddEven(Zero(d), Set(d, uint64_t{1}));
+    const Vec<D> added = Add(v, k1);
+    const Mask<D> overflowed = Lt(added, v);  // false, overflowed
+    // overflowed? 1 : 0, 0
+    const Vec<D> adjust = ShiftLeftLanes<1>(IfThenElseZero(overflowed, k1));
+    return Add(added, adjust);
   }
 };
 
 // Base class shared between OrderAscendingKV128, OrderDescendingKV128.
 struct KeyValue128 : public KeyAny128 {
-  // What type to pass to Sorter::operator().
+  // True indicates only part of the key (the more significant lane) should be
+  // compared. KV stands for key-value.
+  static constexpr bool IsKV() { return true; }
+
+  // What type to pass to VQSort.
   using KeyType = K64V64;
 
-  std::string KeyString() const { return "KV128"; }
+  const char* KeyString() const { return "KV128"; }
 
   template <class D>
   HWY_INLINE Mask<D> EqualKeys(D d, Vec<D> a, Vec<D> b) const {
     return Eq128Upper(d, a, b);
   }
 
-  HWY_INLINE bool Equal1(const LaneType* a, const LaneType* b) {
+  template <class D>
+  HWY_INLINE Mask<D> NotEqualKeys(D d, Vec<D> a, Vec<D> b) const {
+    return Ne128Upper(d, a, b);
+  }
+
+  // Only count differences in the actual key, not the value.
+  template <class D>
+  HWY_INLINE bool NoKeyDifference(D /*tag*/, Vec<D> diff) const {
+    // Must avoid floating-point comparisons (for -0)
+    const RebindToUnsigned<D> du;
+    const Vec<decltype(du)> zero = Zero(du);
+    const Vec<decltype(du)> keys = OddEven(diff, zero);  // clear values
+    return AllTrue(du, Eq(BitCast(du, keys), zero));
+  }
+
+  HWY_INLINE bool Equal1(const LaneType* a, const LaneType* b) const {
     return a[1] == b[1];
   }
+
+  // Returns vector with only the top half of each block valid. This allows
+  // fusing the "replicate upper to lower half" step with a subsequent permute.
+  template <class Order, class D>
+  HWY_INLINE HWY_MAYBE_UNUSED Vec<D> CompareTop(D d, Vec<D> a, Vec<D> b) const {
+    // Only the upper lane of each block is a key, and only that lane is
+    // required to be valid, so comparing all lanes is sufficient.
+    return VecFromMask(d, Order().CompareLanes(a, b));
+  }
 };
 
 struct OrderAscendingKV128 : public KeyValue128 {
   using Order = SortAscending;
+  using OrderForSortingNetwork = OrderAscending128;
 
   HWY_INLINE bool Compare1(const LaneType* a, const LaneType* b) {
     return a[1] < b[1];
@@ -283,7 +350,7 @@ struct OrderAscendingKV128 : public KeyValue128 {
     return Max128Upper(d, a, b);
   }
 
-  // Same as for regular lanes because 128-bit lanes are u64.
+  // Same as for regular lanes because 128-bit keys are u64.
   template <class D>
   HWY_INLINE Vec<D> FirstValue(D d) const {
     return Set(d, hwy::LowestValue<TFromD<D> >());
@@ -296,13 +363,14 @@ struct OrderAscendingKV128 : public KeyValue128 {
 
   template <class D>
   HWY_INLINE Vec<D> PrevValue(D d, Vec<D> v) const {
-    const Vec<D> k1 = OddEven(Zero(d), Set(d, 1));
+    const Vec<D> k1 = OddEven(Set(d, uint64_t{1}), Zero(d));
     return Sub(v, k1);
   }
 };
 
 struct OrderDescendingKV128 : public KeyValue128 {
   using Order = SortDescending;
+  using OrderForSortingNetwork = OrderDescending128;
 
   HWY_INLINE bool Compare1(const LaneType* a, const LaneType* b) {
     return b[1] < a[1];
@@ -329,7 +397,7 @@ struct OrderDescendingKV128 : public KeyValue128 {
     return Min128Upper(d, a, b);
   }
 
-  // Same as for regular lanes because 128-bit lanes are u64.
+  // Same as for regular lanes because 128-bit keys are u64.
   template <class D>
   HWY_INLINE Vec<D> FirstValue(D d) const {
     return Set(d, hwy::HighestValue<TFromD<D> >());
@@ -342,47 +410,50 @@ struct OrderDescendingKV128 : public KeyValue128 {
 
   template <class D>
   HWY_INLINE Vec<D> PrevValue(D d, Vec<D> v) const {
-    const Vec<D> k1 = OddEven(Zero(d), Set(d, 1));
+    const Vec<D> k1 = OddEven(Set(d, uint64_t{1}), Zero(d));
     return Add(v, k1);
   }
 };
 
+// We want to swap 2 u128, i.e. 4 u64 lanes, based on the 0 or FF..FF mask in
+// the most-significant of those lanes (the result of CompareTop), so
+// replicate it 4x. Only called for >= 256-bit vectors.
+
+#if HWY_TARGET <= HWY_AVX3
+template <class V, HWY_IF_V_SIZE_V(V, 64)>
+HWY_INLINE V ReplicateTop4x(V v) {
+  return V{_mm512_permutex_epi64(v.raw, _MM_SHUFFLE(3, 3, 3, 3))};
+}
+#endif  // HWY_TARGET <= HWY_AVX3
+
+#if HWY_TARGET <= HWY_AVX2
+
+template <class V, HWY_IF_V_SIZE_V(V, 32)>
+HWY_INLINE V ReplicateTop4x(V v) {
+  return V{_mm256_permute4x64_epi64(v.raw, _MM_SHUFFLE(3, 3, 3, 3))};
+}
+
+#else  // HWY_TARGET > HWY_AVX2
+
+template <class V>
+HWY_INLINE V ReplicateTop4x(V v) {
+#if HWY_TARGET == HWY_SVE_256
+  return svdup_lane_u64(v, 3);
+#else
+  HWY_ALIGN static constexpr uint64_t kIndices[8] = {3, 3, 3, 3, 7, 7, 7, 7};
+  const ScalableTag<uint64_t> d;
+  return TableLookupLanes(v, SetTableIndices(d, kIndices));
+#endif
+}
+
+#endif  // HWY_TARGET <= HWY_AVX2
+
 // Shared code that depends on Order.
 template <class Base>
-class Traits128 : public Base {
-  // Special case for >= 256 bit vectors
-#if HWY_TARGET <= HWY_AVX2 || HWY_TARGET == HWY_SVE_256
-  // Returns vector with only the top u64 lane valid. Useful when the next step
-  // is to replicate the mask anyway.
-  template <class D>
-  HWY_INLINE HWY_MAYBE_UNUSED Vec<D> CompareTop(D d, Vec<D> a, Vec<D> b) const {
-    const Base* base = static_cast<const Base*>(this);
-    const Mask<D> eqHL = Eq(a, b);
-    const Vec<D> ltHL = VecFromMask(d, base->CompareLanes(a, b));
-#if HWY_TARGET == HWY_SVE_256
-    return IfThenElse(eqHL, DupEven(ltHL), ltHL);
-#else
-    const Vec<D> ltLX = ShiftLeftLanes<1>(ltHL);
-    return OrAnd(ltHL, VecFromMask(d, eqHL), ltLX);
-#endif
-  }
+struct Traits128 : public Base {
+  using TraitsForSortingNetwork =
+      Traits128<typename Base::OrderForSortingNetwork>;
 
-  // We want to swap 2 u128, i.e. 4 u64 lanes, based on the 0 or FF..FF mask in
-  // the most-significant of those lanes (the result of CompareTop), so
-  // replicate it 4x. Only called for >= 256-bit vectors.
-  template <class V>
-  HWY_INLINE V ReplicateTop4x(V v) const {
-#if HWY_TARGET == HWY_SVE_256
-    return svdup_lane_u64(v, 3);
-#elif HWY_TARGET <= HWY_AVX3
-    return V{_mm512_permutex_epi64(v.raw, _MM_SHUFFLE(3, 3, 3, 3))};
-#else  // AVX2
-    return V{_mm256_permute4x64_epi64(v.raw, _MM_SHUFFLE(3, 3, 3, 3))};
-#endif
-  }
-#endif  // HWY_TARGET
-
- public:
   template <class D>
   HWY_INLINE Vec<D> FirstOfLanes(D d, Vec<D> v,
                                  TFromD<D>* HWY_RESTRICT buf) const {
@@ -419,40 +490,28 @@ class Traits128 : public Base {
     b = IfThenElse(lt, b, a_copy);
   }
 
-  // Conditionally swaps even-numbered lanes with their odd-numbered neighbor.
+  // Conditionally swaps even-numbered keys with their odd-numbered neighbor.
   template <class D>
   HWY_INLINE Vec<D> SortPairsDistance1(D d, Vec<D> v) const {
     const Base* base = static_cast<const Base*>(this);
     Vec<D> swapped = base->ReverseKeys2(d, v);
-
-#if HWY_TARGET <= HWY_AVX2 || HWY_TARGET == HWY_SVE_256
-    const Vec<D> select = ReplicateTop4x(CompareTop(d, v, swapped));
-    return IfVecThenElse(select, swapped, v);
-#else
-    Sort2(d, v, swapped);
-    return base->OddEvenKeys(swapped, v);
-#endif
+    const Vec<D> cmpHx = base->template CompareTop<Base>(d, v, swapped);
+    return IfVecThenElse(ReplicateTop4x(cmpHx), swapped, v);
   }
 
-  // Swaps with the vector formed by reversing contiguous groups of 4 keys.
+  // Swaps with the vector formed by reversing contiguous groups of four 128-bit
+  // keys, which implies 512-bit vectors (we do not support more than that).
   template <class D>
   HWY_INLINE Vec<D> SortPairsReverse4(D d, Vec<D> v) const {
     const Base* base = static_cast<const Base*>(this);
     Vec<D> swapped = base->ReverseKeys4(d, v);
 
-    // Only specialize for AVX3 because this requires 512-bit vectors.
-#if HWY_TARGET <= HWY_AVX3
-    const Vec512<uint64_t> outHx = CompareTop(d, v, swapped);
+    const Vec<D> cmpHx = base->template CompareTop<Base>(d, v, swapped);
     // Similar to ReplicateTop4x, we want to gang together 2 comparison results
     // (4 lanes). They are not contiguous, so use permute to replicate 4x.
-    alignas(64) uint64_t kIndices[8] = {7, 7, 5, 5, 5, 5, 7, 7};
-    const Vec512<uint64_t> select =
-        TableLookupLanes(outHx, SetTableIndices(d, kIndices));
+    HWY_ALIGN uint64_t kIndices[8] = {7, 7, 5, 5, 5, 5, 7, 7};
+    const Vec<D> select = TableLookupLanes(cmpHx, SetTableIndices(d, kIndices));
     return IfVecThenElse(select, swapped, v);
-#else
-    Sort2(d, v, swapped);
-    return base->OddEvenPairs(d, swapped, v);
-#endif
   }
 
   // Conditionally swaps lane 0 with 4, 1 with 5 etc.
