@@ -13,32 +13,21 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "hwy/contrib/sort/vqsort.h"
+#include "hwy/contrib/sort/vqsort.h"  // VQSort
 
 #undef HWY_TARGET_INCLUDE
 #define HWY_TARGET_INCLUDE "hwy/contrib/sort/vqsort_128a.cc"
 #include "hwy/foreach_target.h"  // IWYU pragma: keep
 
 // After foreach_target
-#include "hwy/contrib/sort/traits128-inl.h"
 #include "hwy/contrib/sort/vqsort-inl.h"
 
 HWY_BEFORE_NAMESPACE();
 namespace hwy {
 namespace HWY_NAMESPACE {
 
-void Sort128Asc(uint64_t* HWY_RESTRICT keys, size_t num,
-                uint64_t* HWY_RESTRICT buf) {
-#if VQSORT_ENABLED
-  SortTag<uint64_t> d;
-  detail::SharedTraits<detail::Traits128<detail::OrderAscending128>> st;
-  Sort(d, st, keys, num, buf);
-#else
-  (void) keys;
-  (void) num;
-  (void) buf;
-  HWY_ASSERT(0);
-#endif
+void Sort128Asc(uint128_t* HWY_RESTRICT keys, size_t num) {
+  return VQSortStatic(keys, num, SortAscending());
 }
 
 // NOLINTNEXTLINE(google-readability-namespace-comments)
@@ -52,10 +41,8 @@ namespace {
 HWY_EXPORT(Sort128Asc);
 }  // namespace
 
-void Sorter::operator()(uint128_t* HWY_RESTRICT keys, size_t n,
-                        SortAscending) const {
-  HWY_DYNAMIC_DISPATCH(Sort128Asc)
-  (reinterpret_cast<uint64_t*>(keys), n * 2, Get<uint64_t>());
+void VQSort(uint128_t* HWY_RESTRICT keys, size_t n, SortAscending) {
+  HWY_DYNAMIC_DISPATCH(Sort128Asc)(keys, n);
 }
 
 }  // namespace hwy
