@@ -21,7 +21,7 @@
 
 #undef HWY_TARGET_INCLUDE
 #define HWY_TARGET_INCLUDE "highway_test.cc"
-#include "hwy/foreach_target.h"    // IWYU pragma: keep
+#include "hwy/foreach_target.h"  // IWYU pragma: keep
 #include "hwy/highway.h"
 #include "hwy/nanobenchmark.h"  // Unpredictable1
 #include "hwy/tests/test_util-inl.h"
@@ -289,8 +289,8 @@ HWY_INLINE void AssertNaN(D d, VecArg<V> v, const char* file, int line) {
     Abort(file, line,
           "Expected %s NaN, got %E (bytes %02x %02x %02x %02x %02x %02x %02x "
           "%02x)",
-          type_name.c_str(), lane, bytes[0], bytes[1], bytes[2], bytes[3],
-          bytes[4], bytes[5], bytes[6], bytes[7]);
+          type_name.c_str(), static_cast<double>(lane), bytes[0], bytes[1],
+          bytes[2], bytes[3], bytes[4], bytes[5], bytes[6], bytes[7]);
   }
 }
 
@@ -384,25 +384,19 @@ struct TestNaN {
 #endif
     HWY_ASSERT_NAN(d, Min(nan, nan));
     HWY_ASSERT_NAN(d, Max(nan, nan));
-  }
-};
 
-// For functions only available for float32
-struct TestF32NaN {
-  template <class T, class D>
-  HWY_NOINLINE void operator()(T /*unused*/, D d) {
-    const Vec<D> v1 = Set(d, static_cast<T>(Unpredictable1()));
-    const Vec<D> nan = IfThenElse(Eq(v1, Set(d, T{1})), NaN(d), v1);
-    HWY_ASSERT_NAN(d, ApproximateReciprocal(nan));
-    HWY_ASSERT_NAN(d, ApproximateReciprocalSqrt(nan));
+    // AbsDiff
     HWY_ASSERT_NAN(d, AbsDiff(nan, v1));
     HWY_ASSERT_NAN(d, AbsDiff(v1, nan));
+
+    // Approximate*
+    HWY_ASSERT_NAN(d, ApproximateReciprocal(nan));
+    HWY_ASSERT_NAN(d, ApproximateReciprocalSqrt(nan));
   }
 };
 
 HWY_NOINLINE void TestAllNaN() {
   ForFloatTypes(ForPartialVectors<TestNaN>());
-  ForPartialVectors<TestF32NaN>()(float());
 }
 
 struct TestIsNaN {
@@ -508,7 +502,7 @@ struct TestDFromV {
   template <class T, class D>
   HWY_NOINLINE void operator()(T /*unused*/, D d) {
     const Vec<D> v0 = Zero(d);
-    using D0 = DFromV<decltype(v0)>;         // not necessarily same as D
+    using D0 = DFromV<decltype(v0)>;              // not necessarily same as D
     const Vec<D> v0b = And(v0, Set(D0(), T{1}));  // vectors can interoperate
     HWY_ASSERT_VEC_EQ(d, v0, v0b);
   }
