@@ -417,7 +417,8 @@ HWY_API Vec128<T, N> ShiftLeft(Vec128<T, N> v) {
   using TU = hwy::MakeUnsigned<T>;
   for (size_t i = 0; i < N; ++i) {
     const TU raw_u = static_cast<TU>(v.raw[i]);
-    v.raw[i] = static_cast<T>(raw_u << kBits);
+    const auto shifted = raw_u << kBits;  // separate line to avoid MSVC warning
+    v.raw[i] = static_cast<T>(shifted);
   }
   return v;
 }
@@ -1355,6 +1356,16 @@ HWY_API VFromD<D> LoadN(D d, const TFromD<D>* HWY_RESTRICT p,
   return v;
 }
 
+template <class D>
+HWY_API VFromD<D> LoadNOr(VFromD<D> no, D d, const TFromD<D>* HWY_RESTRICT p,
+                          size_t max_lanes_to_load) {
+  VFromD<D> v = no;
+  const size_t N = Lanes(d);
+  const size_t num_of_lanes_to_load = HWY_MIN(max_lanes_to_load, N);
+  CopyBytes(p, v.raw, num_of_lanes_to_load * sizeof(TFromD<D>));
+  return v;
+}
+
 // ------------------------------ Store
 
 template <class D>
@@ -1496,13 +1507,82 @@ HWY_API void Stream(VFromD<D> v, D d, TFromD<D>* HWY_RESTRICT aligned) {
 // ConvertTo and DemoteTo with floating-point input and integer output truncate
 // (rounding toward zero).
 
+namespace detail {
+
+template <class ToT, class FromT>
+HWY_INLINE ToT CastValueForF2IConv(hwy::UnsignedTag /* to_type_tag */,
+                                   FromT val) {
+  // Prevent ubsan errors when converting float to narrower integer
+
+  // If LimitsMax<ToT>() can be exactly represented in FromT,
+  // kSmallestOutOfToTRangePosVal is equal to LimitsMax<ToT>().
+
+  // Otherwise, if LimitsMax<ToT>() cannot be exactly represented in FromT,
+  // kSmallestOutOfToTRangePosVal is equal to LimitsMax<ToT>() + 1, which can
+  // be exactly represented in FromT.
+  constexpr FromT kSmallestOutOfToTRangePosVal =
+      (sizeof(ToT) * 8 <= static_cast<size_t>(MantissaBits<FromT>()) + 1)
+          ? static_cast<FromT>(LimitsMax<ToT>())
+          : static_cast<FromT>(
+                static_cast<FromT>(ToT{1} << (sizeof(ToT) * 8 - 1)) * FromT(2));
+
+  if (std::signbit(val)) {
+    return ToT{0};
+  } else if (std::isinf(val) || val >= kSmallestOutOfToTRangePosVal) {
+    return LimitsMax<ToT>();
+  } else {
+    return static_cast<ToT>(val);
+  }
+}
+
+template <class ToT, class FromT>
+HWY_INLINE ToT CastValueForF2IConv(hwy::SignedTag /* to_type_tag */,
+                                   FromT val) {
+  // Prevent ubsan errors when converting float to narrower integer
+
+  // If LimitsMax<ToT>() can be exactly represented in FromT,
+  // kSmallestOutOfToTRangePosVal is equal to LimitsMax<ToT>().
+
+  // Otherwise, if LimitsMax<ToT>() cannot be exactly represented in FromT,
+  // kSmallestOutOfToTRangePosVal is equal to -LimitsMin<ToT>(), which can
+  // be exactly represented in FromT.
+  constexpr FromT kSmallestOutOfToTRangePosVal =
+      (sizeof(ToT) * 8 <= static_cast<size_t>(MantissaBits<FromT>()) + 2)
+          ? static_cast<FromT>(LimitsMax<ToT>())
+          : static_cast<FromT>(-static_cast<FromT>(LimitsMin<ToT>()));
+
+  if (std::isinf(val) || std::fabs(val) >= kSmallestOutOfToTRangePosVal) {
+    return std::signbit(val) ? LimitsMin<ToT>() : LimitsMax<ToT>();
+  } else {
+    return static_cast<ToT>(val);
+  }
+}
+
+template <class ToT, class ToTypeTag, class FromT>
+HWY_INLINE ToT CastValueForPromoteTo(ToTypeTag /* to_type_tag */, FromT val) {
+  return static_cast<ToT>(val);
+}
+
+template <class ToT>
+HWY_INLINE ToT CastValueForPromoteTo(hwy::SignedTag to_type_tag, float val) {
+  return CastValueForF2IConv<ToT>(to_type_tag, val);
+}
+
+template <class ToT>
+HWY_INLINE ToT CastValueForPromoteTo(hwy::UnsignedTag to_type_tag, float val) {
+  return CastValueForF2IConv<ToT>(to_type_tag, val);
+}
+
+}  // namespace detail
+
 template <class DTo, typename TFrom, HWY_IF_NOT_SPECIAL_FLOAT(TFrom)>
 HWY_API VFromD<DTo> PromoteTo(DTo d, Vec128<TFrom, HWY_MAX_LANES_D(DTo)> from) {
   static_assert(sizeof(TFromD<DTo>) > sizeof(TFrom), "Not promoting");
   VFromD<DTo> ret;
   for (size_t i = 0; i < MaxLanes(d); ++i) {
     // For bits Y > X, floatX->floatY and intX->intY are always representable.
-    ret.raw[i] = static_cast<TFromD<DTo>>(from.raw[i]);
+    ret.raw[i] = detail::CastValueForPromoteTo<TFromD<DTo>>(
+        hwy::TypeTag<TFromD<DTo>>(), from.raw[i]);
   }
   return ret;
 }
@@ -1524,18 +1604,13 @@ HWY_API VFromD<D> DemoteTo(D d, VFromD<Rebind<double, D>> from) {
   }
   return ret;
 }
-template <class D, HWY_IF_I32_D(D)>
+template <class D, HWY_IF_UI32_D(D)>
 HWY_API VFromD<D> DemoteTo(D d, VFromD<Rebind<double, D>> from) {
   VFromD<D> ret;
   for (size_t i = 0; i < MaxLanes(d); ++i) {
-    // Prevent ubsan errors when converting int32_t to narrower integer/int32_t
-    if (std::isinf(from.raw[i]) ||
-        std::fabs(from.raw[i]) > static_cast<double>(HighestValue<int32_t>())) {
-      ret.raw[i] = std::signbit(from.raw[i]) ? LowestValue<int32_t>()
-                                             : HighestValue<int32_t>();
-      continue;
-    }
-    ret.raw[i] = static_cast<int32_t>(from.raw[i]);
+    // Prevent ubsan errors when converting double to narrower integer/int32_t
+    ret.raw[i] = detail::CastValueForF2IConv<TFromD<D>>(
+        hwy::TypeTag<TFromD<D>>(), from.raw[i]);
   }
   return ret;
 }
@@ -1571,6 +1646,21 @@ HWY_API VFromD<DTo> DemoteTo(DTo /* tag */, Vec128<TFrom, N> from) {
   return ret;
 }
 
+template <class DTo, typename TFrom, size_t N, HWY_IF_UI64(TFrom),
+          HWY_IF_F32_D(DTo)>
+HWY_API VFromD<DTo> DemoteTo(DTo /* tag */, Vec128<TFrom, N> from) {
+  using TTo = TFromD<DTo>;
+  static_assert(sizeof(TTo) < sizeof(TFrom), "Not demoting");
+
+  VFromD<DTo> ret;
+  for (size_t i = 0; i < N; ++i) {
+    // int64_t/uint64_t to float: okay to cast to float as an int64_t/uint64_t
+    // value is always within the range of a float
+    ret.raw[i] = static_cast<TTo>(from.raw[i]);
+  }
+  return ret;
+}
+
 template <class DBF16, HWY_IF_BF16_D(DBF16), class VF32>
 HWY_API VFromD<DBF16> ReorderDemote2To(DBF16 dbf16, VF32 a, VF32 b) {
   const Repartition<uint32_t, decltype(dbf16)> du32;
@@ -1687,17 +1777,10 @@ HWY_API VFromD<DTo> ConvertTo(hwy::FloatTag /*tag*/, DTo /*tag*/,
   static_assert(sizeof(ToT) == sizeof(TFrom), "Should have same size");
   VFromD<DTo> ret;
   constexpr size_t N = HWY_MAX_LANES_D(DTo);
+
   for (size_t i = 0; i < N; ++i) {
-    // float## -> int##: return closest representable value. We cannot exactly
-    // represent LimitsMax<ToT> in TFrom, so use double.
-    const double f = static_cast<double>(from.raw[i]);
-    if (std::isinf(from.raw[i]) ||
-        std::fabs(f) > static_cast<double>(LimitsMax<ToT>())) {
-      ret.raw[i] =
-          std::signbit(from.raw[i]) ? LimitsMin<ToT>() : LimitsMax<ToT>();
-      continue;
-    }
-    ret.raw[i] = static_cast<ToT>(from.raw[i]);
+    // float## -> int##: return closest representable value
+    ret.raw[i] = CastValueForF2IConv<ToT>(hwy::TypeTag<ToT>(), from.raw[i]);
   }
   return ret;
 }
