@@ -43,6 +43,7 @@ struct TestBroadcastR {
     for (size_t block = 0; block < N; block += blockN) {
       in_lanes[block + kLane] = static_cast<T>(block + 1);
     }
+    PreventElision(in_lanes[0]);  // workaround for f16x1 failure
     const auto in = Load(d, in_lanes.get());
     for (size_t block = 0; block < N; block += blockN) {
       for (size_t i = 0; i < blockN; ++i) {
@@ -271,15 +272,23 @@ struct TestZipLower {
   }
 };
 
+#if HWY_TARGET == HWY_SCALAR
+template <class Test>
+using ForZipToWideVectors = ForPartialVectors<Test>;
+#else
+template <class Test>
+using ForZipToWideVectors = ForShrinkableVectors<Test>;
+#endif
+
 HWY_NOINLINE void TestAllZipLower() {
-  const ForDemoteVectors<TestZipLower> lower_unsigned;
+  const ForZipToWideVectors<TestZipLower> lower_unsigned;
   lower_unsigned(uint8_t());
   lower_unsigned(uint16_t());
 #if HWY_HAVE_INTEGER64
   lower_unsigned(uint32_t());  // generates u64
 #endif
 
-  const ForDemoteVectors<TestZipLower> lower_signed;
+  const ForZipToWideVectors<TestZipLower> lower_signed;
   lower_signed(int8_t());
   lower_signed(int16_t());
 #if HWY_HAVE_INTEGER64
