@@ -13,14 +13,11 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include <stddef.h>
-#include <stdint.h>
+#include <stdio.h>
 #include <string.h>  // memset
 
 #include <array>  // IWYU pragma: keep
 
-#include "hwy/base.h"
-
 #undef HWY_TARGET_INCLUDE
 #define HWY_TARGET_INCLUDE "tests/compress_test.cc"
 #include "hwy/foreach_target.h"  // IWYU pragma: keep
@@ -37,13 +34,15 @@ namespace HWY_NAMESPACE {
 #if !HWY_PRINT_TABLES || HWY_IDE
 
 template <class D, class DI, typename T = TFromD<D>, typename TI = TFromD<DI>>
-void CheckStored(D d, DI di, size_t expected_pos, size_t actual_pos,
-                 size_t num_to_check, const AlignedFreeUniquePtr<T[]>& in,
+void CheckStored(D d, DI di, const char* op, size_t expected_pos,
+                 size_t actual_pos, size_t num_to_check,
+                 const AlignedFreeUniquePtr<T[]>& in,
                  const AlignedFreeUniquePtr<TI[]>& mask_lanes,
                  const AlignedFreeUniquePtr<T[]>& expected, const T* actual_u,
                  int line) {
   if (expected_pos != actual_pos) {
-    hwy::Abort(__FILE__, line, "Size mismatch for %s: expected %d, actual %d\n",
+    hwy::Abort(__FILE__, line,
+               "%s: size mismatch for %s: expected %d, actual %d\n", op,
                TypeName(T(), Lanes(d)).c_str(), static_cast<int>(expected_pos),
                static_cast<int>(actual_pos));
   }
@@ -51,12 +50,12 @@ void CheckStored(D d, DI di, size_t expected_pos, size_t actual_pos,
   for (size_t i = 0; i < num_to_check; ++i) {
     if (!IsEqual(expected[i], actual_u[i])) {
       const size_t N = Lanes(d);
-      fprintf(stderr, "Mismatch at i=%d of %d, line %d:\n\n",
+      fprintf(stderr, "%s: mismatch at i=%d of %d, line %d:\n\n", op,
               static_cast<int>(i), static_cast<int>(num_to_check), line);
       Print(di, "mask", Load(di, mask_lanes.get()), 0, N);
       Print(d, "in", Load(d, in.get()), 0, N);
-      Print(d, "expect", Load(d, expected.get()), 0, N);
-      Print(d, "actual", Load(d, actual_u), 0, N);
+      Print(d, "expect", Load(d, expected.get()), 0, num_to_check);
+      Print(d, "actual", Load(d, actual_u), 0, num_to_check);
       HWY_ASSERT(false);
     }
   }
@@ -68,10 +67,10 @@ struct TestCompress {
     RandomState rng;
 
     using TI = MakeSigned<T>;  // For mask > 0 comparison
+    using TU = MakeUnsigned<T>;
     const Rebind<TI, D> di;
     const size_t N = Lanes(d);
-
-    const T zero{0};
+    const size_t bits_size = RoundUpTo((N + 7) / 8, 8);
 
     for (int frac : {0, 2, 3}) {
       // For CompressStore
@@ -79,25 +78,28 @@ struct TestCompress {
 
       auto in_lanes = AllocateAligned<T>(N);
       auto mask_lanes = AllocateAligned<TI>(N);
+      auto garbage = AllocateAligned<TU>(N);
       auto expected = AllocateAligned<T>(N);
       auto actual_a = AllocateAligned<T>(misalign + N);
-      T* actual_u = actual_a.get() + misalign;
-
-      const size_t bits_size = RoundUpTo((N + 7) / 8, 8);
       auto bits = AllocateAligned<uint8_t>(bits_size);
+      HWY_ASSERT(in_lanes && mask_lanes && garbage && expected && actual_a &&
+                 bits);
+
+      T* actual_u = actual_a.get() + misalign;
       memset(bits.get(), 0, bits_size);  // for MSAN
 
       // Each lane should have a chance of having mask=true.
       for (size_t rep = 0; rep < AdjustedReps(200); ++rep) {
         size_t expected_pos = 0;
         for (size_t i = 0; i < N; ++i) {
-          const uint64_t bits = Random32(&rng);
+          const uint64_t r = Random32(&rng);
           in_lanes[i] = T();  // cannot initialize float16_t directly.
-          CopyBytes<sizeof(T)>(&bits, &in_lanes[i]);  // not same size
+          CopyBytes<sizeof(T)>(&r, &in_lanes[i]);  // not same size
           mask_lanes[i] = (Random32(&rng) & 1024) ? TI(1) : TI(0);
           if (mask_lanes[i] > 0) {
             expected[expected_pos++] = in_lanes[i];
           }
+          garbage[i] = static_cast<TU>(Random64(&rng));
         }
         size_t num_to_check;
         if (CompressIsPartition<T>::value) {
@@ -124,55 +126,63 @@ struct TestCompress {
         // Compress
         memset(actual_u, 0, N * sizeof(T));
         StoreU(Compress(in, mask), d, actual_u);
-        CheckStored(d, di, expected_pos, expected_pos, num_to_check, in_lanes,
-                    mask_lanes, expected, actual_u, __LINE__);
+        CheckStored(d, di, "Compress", expected_pos, expected_pos, num_to_check,
+                    in_lanes, mask_lanes, expected, actual_u, __LINE__);
 
         // CompressNot
         memset(actual_u, 0, N * sizeof(T));
         StoreU(CompressNot(in, Not(mask)), d, actual_u);
-        CheckStored(d, di, expected_pos, expected_pos, num_to_check, in_lanes,
-                    mask_lanes, expected, actual_u, __LINE__);
+        CheckStored(d, di, "CompressNot", expected_pos, expected_pos,
+                    num_to_check, in_lanes, mask_lanes, expected, actual_u,
+                    __LINE__);
 
         // CompressStore
         memset(actual_u, 0, N * sizeof(T));
         const size_t size1 = CompressStore(in, mask, d, actual_u);
         // expected_pos instead of num_to_check because this op is not
         // affected by CompressIsPartition.
-        CheckStored(d, di, expected_pos, size1, expected_pos, in_lanes,
-                    mask_lanes, expected, actual_u, __LINE__);
+        CheckStored(d, di, "CompressStore", expected_pos, size1, expected_pos,
+                    in_lanes, mask_lanes, expected, actual_u, __LINE__);
 
         // CompressBlendedStore
-        memset(actual_u, 0, N * sizeof(T));
+        memcpy(actual_u, garbage.get(), N * sizeof(T));
         const size_t size2 = CompressBlendedStore(in, mask, d, actual_u);
         // expected_pos instead of num_to_check because this op only writes
         // the mask=true lanes.
-        CheckStored(d, di, expected_pos, size2, expected_pos, in_lanes,
-                    mask_lanes, expected, actual_u, __LINE__);
+        CheckStored(d, di, "CompressBlendedStore", expected_pos, size2,
+                    expected_pos, in_lanes, mask_lanes, expected, actual_u,
+                    __LINE__);
         // Subsequent lanes are untouched.
         for (size_t i = size2; i < N; ++i) {
-          HWY_ASSERT_EQ(zero, actual_u[i]);
+#if HWY_COMPILER_MSVC && HWY_TARGET == HWY_AVX2
+          // TODO(eustas): re-enable when compiler is fixed
+#else
+          HWY_ASSERT_EQ(garbage[i], reinterpret_cast<TU*>(actual_u)[i]);
+#endif
         }
 
         // CompressBits
         memset(actual_u, 0, N * sizeof(T));
         StoreU(CompressBits(in, bits.get()), d, actual_u);
-        CheckStored(d, di, expected_pos, expected_pos, num_to_check, in_lanes,
-                    mask_lanes, expected, actual_u, __LINE__);
+        CheckStored(d, di, "CompressBits", expected_pos, expected_pos,
+                    num_to_check, in_lanes, mask_lanes, expected, actual_u,
+                    __LINE__);
 
         // CompressBitsStore
         memset(actual_u, 0, N * sizeof(T));
         const size_t size3 = CompressBitsStore(in, bits.get(), d, actual_u);
         // expected_pos instead of num_to_check because this op is not
         // affected by CompressIsPartition.
-        CheckStored(d, di, expected_pos, size3, expected_pos, in_lanes,
-                    mask_lanes, expected, actual_u, __LINE__);
+        CheckStored(d, di, "CompressBitsStore", expected_pos, size3,
+                    expected_pos, in_lanes, mask_lanes, expected, actual_u,
+                    __LINE__);
       }  // rep
     }    // frac
   }      // operator()
 };
 
 HWY_NOINLINE void TestAllCompress() {
-  ForUIF163264(ForPartialVectors<TestCompress>());
+  ForAllTypes(ForPartialVectors<TestCompress>());
 }
 
 struct TestCompressBlocks {
@@ -192,6 +202,7 @@ struct TestCompressBlocks {
     auto mask_lanes = AllocateAligned<TI>(N);
     auto expected = AllocateAligned<T>(N);
     auto actual = AllocateAligned<T>(N);
+    HWY_ASSERT(in_lanes && mask_lanes && expected && actual);
 
     // Each lane should have a chance of having mask=true.
     for (size_t rep = 0; rep < AdjustedReps(200); ++rep) {
@@ -230,8 +241,9 @@ struct TestCompressBlocks {
       // CompressBlocksNot
       memset(actual.get(), 0, N * sizeof(T));
       StoreU(CompressBlocksNot(in, Not(mask)), d, actual.get());
-      CheckStored(d, di, expected_pos, expected_pos, num_to_check, in_lanes,
-                  mask_lanes, expected, actual.get(), __LINE__);
+      CheckStored(d, di, "CompressBlocksNot", expected_pos, expected_pos,
+                  num_to_check, in_lanes, mask_lanes, expected, actual.get(),
+                  __LINE__);
     }  // rep
 #endif  // HWY_TARGET == HWY_SCALAR
   }     // operator()
@@ -246,6 +258,34 @@ HWY_NOINLINE void TestAllCompressBlocks() {
 #if HWY_PRINT_TABLES || HWY_IDE
 namespace detail {  // for code folding
 
+void PrintCompress8x8Tables() {
+  printf("======================================= 8x8\n");
+  constexpr size_t N = 8;
+  for (uint64_t code = 0; code < (1ull << N); ++code) {
+    std::array<uint8_t, N> indices{0};
+    size_t pos = 0;
+    // All lanes where mask = true
+    for (size_t i = 0; i < N; ++i) {
+      if (code & (1ull << i)) {
+        indices[pos++] = i;
+      }
+    }
+    // All lanes where mask = false
+    for (size_t i = 0; i < N; ++i) {
+      if (!(code & (1ull << i))) {
+        indices[pos++] = i;
+      }
+    }
+    HWY_ASSERT(pos == N);
+
+    for (size_t i = 0; i < N; ++i) {
+      printf("%d,", indices[i]);
+    }
+    printf(code & 1 ? "//\n" : "/**/");
+  }
+  printf("\n");
+}
+
 void PrintCompress16x8Tables() {
   printf("======================================= 16x8\n");
   constexpr size_t N = 8;  // 128-bit SIMD
@@ -305,11 +345,13 @@ void PrintCompressNot16x8Tables() {
   printf("\n");
 }
 
-// Compressed to nibbles, unpacked via variable right shift
+// Compressed to nibbles, unpacked via variable right shift. Also includes
+// FirstN bits in the nibble MSB.
 void PrintCompress32x8Tables() {
   printf("======================================= 32/64x8\n");
   constexpr size_t N = 8;  // AVX2 or 64-bit AVX3
   for (uint64_t code = 0; code < (1ull << N); ++code) {
+    const size_t count = PopCount(code);
     std::array<uint32_t, N> indices{0};
     size_t pos = 0;
     // All lanes where mask = true
@@ -330,6 +372,10 @@ void PrintCompress32x8Tables() {
     uint64_t packed = 0;
     for (size_t i = 0; i < N; ++i) {
       HWY_ASSERT(indices[i] < N);
+      if (i < count) {
+        indices[i] |= N;
+        HWY_ASSERT(indices[i] < 0x10);
+      }
       packed += indices[i] << (i * 4);
     }
 
@@ -344,6 +390,7 @@ void PrintCompressNot32x8Tables() {
   constexpr size_t N = 8;  // AVX2 or 64-bit AVX3
   for (uint64_t not_code = 0; not_code < (1ull << N); ++not_code) {
     const uint64_t code = ~not_code;
+    const size_t count = PopCount(code);
     std::array<uint32_t, N> indices{0};
     size_t pos = 0;
     // All lanes where mask = true
@@ -364,6 +411,10 @@ void PrintCompressNot32x8Tables() {
     uint64_t packed = 0;
     for (size_t i = 0; i < N; ++i) {
       HWY_ASSERT(indices[i] < N);
+      if (i < count) {
+        indices[i] |= N;
+        HWY_ASSERT(indices[i] < 0x10);
+      }
       packed += indices[i] << (i * 4);
     }
 
@@ -441,6 +492,40 @@ void PrintCompressNot64x4NibbleTables() {
   printf("\n");
 }
 
+void PrintCompressNot64x2NibbleTables() {
+  printf("======================================= Not 64x2Nibble\n");
+  constexpr size_t N = 2;  // 128-bit
+  for (uint64_t not_code = 0; not_code < (1ull << N); ++not_code) {
+    const uint64_t code = ~not_code;
+    std::array<uint32_t, N> indices{0};
+    size_t pos = 0;
+    // All lanes where mask = true
+    for (size_t i = 0; i < N; ++i) {
+      if (code & (1ull << i)) {
+        indices[pos++] = i;
+      }
+    }
+    // All lanes where mask = false
+    for (size_t i = 0; i < N; ++i) {
+      if (!(code & (1ull << i))) {
+        indices[pos++] = i;
+      }
+    }
+    HWY_ASSERT(pos == N);
+
+    // Convert to nibbles
+    uint64_t packed = 0;
+    for (size_t i = 0; i < N; ++i) {
+      HWY_ASSERT(indices[i] < N);
+      packed += indices[i] << (i * 4);
+    }
+
+    HWY_ASSERT(packed < (1ull << (N * 4)));
+    printf("0x%08x,", static_cast<uint32_t>(packed));
+  }
+  printf("\n");
+}
+
 void PrintCompress64x4Tables() {
   printf("======================================= 64x4 uncompressed\n");
   constexpr size_t N = 4;  // SVE_256
@@ -504,11 +589,13 @@ void PrintCompressNot64x4Tables() {
   printf("\n");
 }
 
-// Same as above, but prints pairs of u32 indices (for AVX2)
+// Same as above, but prints pairs of u32 indices (for AVX2). Also includes
+// FirstN bits in the nibble MSB.
 void PrintCompress64x4PairTables() {
   printf("======================================= 64x4 u32 index\n");
   constexpr size_t N = 4;  // AVX2
   for (uint64_t code = 0; code < (1ull << N); ++code) {
+    const size_t count = PopCount(code);
     std::array<size_t, N> indices{0};
     size_t pos = 0;
     // All lanes where mask = true
@@ -530,8 +617,10 @@ void PrintCompress64x4PairTables() {
     // interpreted modulo N. Compression is not worth the extra shift+AND
     // because the table is anyway only 512 bytes.
     for (size_t i = 0; i < N; ++i) {
-      printf("%d, %d, ", static_cast<int>(2 * indices[i] + 0),
-             static_cast<int>(2 * indices[i]) + 1);
+      const int first_n_bit = i < count ? 8 : 0;
+      const int low = static_cast<int>(2 * indices[i]) + first_n_bit;
+      HWY_ASSERT(low < 0x10);
+      printf("%d, %d, ", low, low + 1);
     }
   }
   printf("\n");
@@ -542,6 +631,7 @@ void PrintCompressNot64x4PairTables() {
   constexpr size_t N = 4;  // AVX2
   for (uint64_t not_code = 0; not_code < (1ull << N); ++not_code) {
     const uint64_t code = ~not_code;
+    const size_t count = PopCount(code);
     std::array<size_t, N> indices{0};
     size_t pos = 0;
     // All lanes where mask = true
@@ -563,8 +653,10 @@ void PrintCompressNot64x4PairTables() {
     // interpreted modulo N. Compression is not worth the extra shift+AND
     // because the table is anyway only 512 bytes.
     for (size_t i = 0; i < N; ++i) {
-      printf("%d, %d, ", static_cast<int>(2 * indices[i] + 0),
-             static_cast<int>(2 * indices[i]) + 1);
+      const int first_n_bit = i < count ? 8 : 0;
+      const int low = static_cast<int>(2 * indices[i]) + first_n_bit;
+      HWY_ASSERT(low < 0x10);
+      printf("%d, %d, ", low, low + 1);
     }
   }
   printf("\n");
@@ -703,6 +795,7 @@ HWY_NOINLINE void PrintTables() {
   detail::PrintCompressNot32x8Tables();
   detail::PrintCompress64x4NibbleTables();
   detail::PrintCompressNot64x4NibbleTables();
+  detail::PrintCompressNot64x2NibbleTables();
   detail::PrintCompress64x4Tables();
   detail::PrintCompressNot64x4Tables();
   detail::PrintCompress32x4Tables();
@@ -712,6 +805,7 @@ HWY_NOINLINE void PrintTables() {
   detail::PrintCompress64x4PairTables();
   detail::PrintCompressNot64x4PairTables();
   detail::PrintCompress16x8Tables();
+  detail::PrintCompress8x8Tables();
   detail::PrintCompressNot16x8Tables();
 #endif
 }
